var test = require('tape');
var React = require('react');
var PropTypes = require('prop-types');
var ReactTestUtils = require('react-dom/test-utils');
var Styletron = require('styletron-server');

var styled = require('../styled');
var Provider = require('../provider');

test('provider provides instance', function (t) {
  var mockInstance = {};
  var MockComponent = function (props, context) {
    t.equal(context.styletron, mockInstance, 'styletron instance override provided');
    return React.createElement('div');
  };
  MockComponent.contextTypes = {styletron: PropTypes.object};
  ReactTestUtils.renderIntoDocument(React.createElement(Provider, {
    styletron: mockInstance
  }, React.createElement(MockComponent)));
  t.end();
});

test('props passed to styled function', function (t) {
  t.plan(1);
  var props = {
    prop1: 'foo'
  };
  var Widget = styled('div', function (props) {
    t.deepEqual(props, props, 'props accessible in style fn');
    return {};
  });
  var styletron = new Styletron();
  ReactTestUtils.renderIntoDocument(
    React.createElement(Provider, {styletron: styletron},
      React.createElement(Widget, props))
  );
});

test('styled applies styles', function (t) {
  var Widget = styled('div', function () {
    return {color: 'red'};
  });
  var styletron = new Styletron();
  var output = ReactTestUtils.renderIntoDocument(
    React.createElement(Provider, {styletron: styletron},
      React.createElement(Widget))
  );
  var div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.className, 'a', 'styletron classes');
  t.equal(styletron.getCss(), '.a{color:red}');
  t.end();
});

test('styled applies static styles', function (t) {
  var Widget = styled('div', {color: 'red'});
  var styletron = new Styletron();
  var output = ReactTestUtils.renderIntoDocument(
    React.createElement(Provider, {styletron: styletron},
      React.createElement(Widget))
  );
  var div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.className, 'a', 'matches expected styletron classes');
  t.equal(styletron.getCss(), '.a{color:red}');
  t.end();
});

test('styled passes through valid props', function (t) {
  var Widget = styled('div', {color: 'red'});
  var styletron = new Styletron();
  var output = ReactTestUtils.renderIntoDocument(
    React.createElement(Provider, {styletron: styletron},
      React.createElement(Widget, {
        'data-bar': 'bar'
      }))
  );
  var div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.getAttribute('data-bar'), 'bar', 'valid attribute prop passed through');
  t.end();
});

test('styled composition', function (t) {
  var Widget = styled('div', {color: 'red', 'display': 'inline'});
  var SuperWidget = styled(Widget, {display: 'block', background: 'black'});
  var styletron = new Styletron();
  var output = ReactTestUtils.renderIntoDocument(
    React.createElement(Provider, {styletron: styletron},
      React.createElement(SuperWidget))
  );
  var div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.className, 'a b c', 'matches expected styletron classes');
  t.equal(styletron.getCss(), '.a{color:red}.b{display:block}.c{background:black}');
  t.end();
});

test('styled component', function (t) {
  var Widget = function (ref) {
    var className = ref.className;

    return React.createElement('div', {className: className});
  };
  var SuperWidget = styled(Widget, {color: 'red'});
  var styletron = new Styletron();
  var output = ReactTestUtils.renderIntoDocument(
    React.createElement(Provider, {styletron: styletron},
      React.createElement(SuperWidget))
  );
  var div = ReactTestUtils.findRenderedDOMComponentWithTag(output, 'div');
  t.equal(div.className, 'a', 'matches expected styletron classes');
  t.equal(styletron.getCss(), '.a{color:red}');
  t.end();
});

test('innerRef works', function (t) {
  t.plan(1);

  var Widget = styled('button', {color: 'red'});
  var styletron = new Styletron();

  var TestComponent = (function (superclass) {
    function TestComponent () {
      superclass.apply(this, arguments);
    }

    if ( superclass ) TestComponent.__proto__ = superclass;
    TestComponent.prototype = Object.create( superclass && superclass.prototype );
    TestComponent.prototype.constructor = TestComponent;

    TestComponent.prototype.componentDidMount = function componentDidMount () {
      t.ok(this.widgetInner instanceof HTMLButtonElement, 'is button');
    };

    TestComponent.prototype.render = function render () {
      var this$1 = this;

      return React.createElement(Widget, {innerRef: function (c) {
        this$1.widgetInner = c;
      }});
    };

    return TestComponent;
  }(React.Component));

  ReactTestUtils.renderIntoDocument(
    React.createElement(Provider, {styletron: styletron},
      React.createElement(TestComponent))
  );
});

test('innerRef not passed', function (t) {
  t.plan(2);

  var InnerComponent = (function (superclass) {
    function InnerComponent () {
      superclass.apply(this, arguments);
    }

    if ( superclass ) InnerComponent.__proto__ = superclass;
    InnerComponent.prototype = Object.create( superclass && superclass.prototype );
    InnerComponent.prototype.constructor = InnerComponent;

    InnerComponent.prototype.render = function render () {
      t.deepEqual(this.props, {
        className: 'a',
        foo: 'bar'
      }, 'props match expected');
      return React.createElement( 'button', null, "InnerComponent" );
    };

    return InnerComponent;
  }(React.Component));

  var Widget = styled(InnerComponent, {color: 'red'});
  var styletron = new Styletron();

  var TestComponent = (function (superclass) {
    function TestComponent () {
      superclass.apply(this, arguments);
    }

    if ( superclass ) TestComponent.__proto__ = superclass;
    TestComponent.prototype = Object.create( superclass && superclass.prototype );
    TestComponent.prototype.constructor = TestComponent;

    TestComponent.prototype.componentDidMount = function componentDidMount () {
      t.ok(
        ReactTestUtils.isCompositeComponentWithType(this.widgetInner, InnerComponent),
        'is InnerComponent'
      );
    };

    TestComponent.prototype.render = function render () {
      var this$1 = this;

      return React.createElement(Widget, {
        foo: 'bar',
        innerRef: function (c) {
          this$1.widgetInner = c;
        }
      });
    };

    return TestComponent;
  }(React.Component));

  ReactTestUtils.renderIntoDocument(
    React.createElement(Provider, {styletron: styletron},
      React.createElement(TestComponent))
  );
});
